package tareas.tarea2.electronica;

import java.awt.Point;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

/**
 * Clase hija de Componente que implementa un punto de tierra.
 * @author Janther
 */
public class GND extends Componente {

    /**
     * Numero de nodos en el Componente.
     */
    private final int numNodos = 1;
    /**
     * Nodo del Componente.
     */
    private Nodo[] nodos = new Nodo[numNodos];
    /**
     * Numero de vertices en la Forma.
     */
    private final int numVertices = 8;
    /**
     * Vertices del Componente.
     */
    private Point[] vertices = new Point[numVertices];
    /**
     * Tamao del Componente.
     */
    private final double size = 24;
    /**
     * El area para seleccionar el componente.
     */
    private Rectangle2D area;
    /**
     * Color de la forma.
     */
    private Color c;

    /**
     * Constructor por omision del componente, lo genera en la esquina superior
     * izquierda del panel.
     */
    public GND() {
        this(new Point());
        final int maxDiv = 6;
        trasladar(size * 3 / maxDiv, 0);
    }
    /**
     * Constructor por omision del componente, lo genera en la esquina superior
     * izquierda del panel.
     * @param inicio Es el punto inicial del dibujo del punto de tierra.
     */
    public GND(Point inicio) {
        double x = inicio.getX();
        double y = inicio.getY();
        for (int i = 0; i < numVertices; i++) {
            vertices[i] = new Point();
        }
        final int maxDiv = 6;
        vertices[0].setLocation(x, y);
        vertices[1].setLocation(x, y + size * 3 / maxDiv);
        vertices[2].setLocation(x - size * 3 / maxDiv, y + size * 3 / maxDiv);
        vertices[3].setLocation(x + size * 3 / maxDiv, y + size * 3 / maxDiv);
        vertices[4].setLocation(x - size * 2 / maxDiv, y + size * 4 / maxDiv);
        vertices[5].setLocation(x + size * 2 / maxDiv, y + size * 4 / maxDiv);
        vertices[6].setLocation(x - size * 1 / maxDiv, y + size * 5 / maxDiv);
        vertices[7].setLocation(x + size * 1 / maxDiv, y + size * 5 / maxDiv);
        nodos[0] = new Nodo(vertices[0]);
        setArea();
    }

    /**
     * Constructor de la clase que sirve para clonar.
     * @param esquinas Esquinas que dibujaran la tierra.
     */
    public GND(Point[] esquinas) {
        for (int i = 0; i < esquinas.length; i++) {
            vertices[i] = (Point) esquinas[i].clone();
        }
        nodos[0] = new Nodo(vertices[0]);
        setArea();
    }

    /**
     * Setea el area seleccionable del componente en base a los puntos que lo
     * conforman.
     */
    private void setArea() {
        double maxX = vertices[0].getX();
        double maxY = vertices[0].getY();
        double minX = vertices[0].getX();
        double minY = vertices[0].getY();
        for (int i = 1; i < numVertices; i++) {
            if (vertices[i].getX() < minX) {
                minX = vertices[i].getX();
            }
            if (vertices[i].getX() > maxX) {
                maxX = vertices[i].getX();
            }
            if (vertices[i].getY() < minY) {
                minY = vertices[i].getY();
            }
            if (vertices[i].getY() > maxY) {
                maxY = vertices[i].getY();
            }
        }
        double ancho = maxX - minX + 1;
        double alto = maxY - minY + 1;
        area = new Rectangle2D.Double(minX, minY, ancho, alto);
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) {
        /*
         * Se traslado coda punto del Punto de Tierra con respecto al
         * punto "centro" en funcion del factor.
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * factor + centro.getX();
            ny = (vertices[i].getY() - centro.getY()) * factor + centro.getY();
            vertices[i].setLocation(nx, ny);
        }
        nodos[0].escalar(centro, factor);
        setArea();
    }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Se uso la transformacion rotacional para un vector.
         * http://es.wikipedia.org/wiki/Rotaci%C3%B3n
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * Math.cos(deltaAngulo)
                - (vertices[i].getY() - centro.getY()) * Math.sin(deltaAngulo);
            ny = (vertices[i].getX() - centro.getX()) * Math.sin(deltaAngulo)
                + (vertices[i].getY() - centro.getY()) * Math.cos(deltaAngulo);
            vertices[i].setLocation(nx + centro.getX(), ny + centro.getY());
        }
        nodos[0].rotar(centro, deltaAngulo);
        setArea();
    }

    /** Rota la forma respecto a su centro en 90 grados.
     */
    public void rotar() {
        double nx = area.getX() + (area.getWidth() - 1) / 2;
        double ny = area.getY() + (area.getHeight() - 1) / 2;
        Point centro = new Point();
        centro.setLocation(nx, ny);
        rotar(centro, Math.PI / 2);
    }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Se translado cada coordenada en su delta respectivo.
         */
        for (int i = 0; i < numVertices; i++) {
            vertices[i].setLocation(vertices[i].getX() + deltaX,
                                    vertices[i].getY() + deltaY);
        }
        nodos[0].trasladar(deltaX, deltaY);
        setArea();
    }

    /**
     * Verifica si un punto esta dentro de el area seleccionable de la forma.
     * @param p Punto a verificar.
     * @return Verdadero si el punto esta dentro del area.
     */
    public boolean isSelected(Point p) {
        if (area.contains(p) || nodos[0].isSelected(p)) {
            c = Color.RED;
            return true;
        }
        c = Color.BLACK;
        return false;
    }

    /** Dibuja la Forma en el Graphics2D.
     * @param g2d Grafico en el cual se incluira el dibujo.
     */
    public void paintComponent(Graphics2D g2d) {
        g2d.setColor(c);
        for (int i = 1; i < numVertices; i = i + 2) {
            g2d.draw(new Line2D.Double(vertices[i - 1], vertices[i]));
        }
        nodos[0].paintComponent(g2d);
    }

    /** Retorna los nodos relacionados a este Componente.
     * @return Arreglo de nodos.
     */
    public Nodo[] getNodos() {
        return nodos;
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new GND(vertices);
    }

    /** Retorna el punto que basta para localizar la coordenada de un
     * componente, no asi su orientacion.
     * @return Punto con las coordenadas del componente.
     */
    public Point getPunto() {
        return vertices[0];
    }
}
