package tareas.tarea2.electronica;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 * Clase hija de Componente que implementa un Nodo.
 * @author Janther
 */
public class Nodo extends Componente {

    /**
     * Es el punto central del nodo.
     */
    private Point a = new Point();
    /**
     * El area para seleccionar el componente.
     */
    private Rectangle2D area;
    /**
     * Color de la forma.
     */
    private Color c;
    /**
     * El arreglo de cables unidos a este nodo.
     */
    private ArrayList < Cable > cables = new ArrayList < Cable >();

    /**
     * Constructor de la clase.
     * @param inicio el punto de inicio de la Resistencia.
     */
    public Nodo(final Point inicio) {
        a = (Point) inicio.clone();
        setArea();
    }

    /**
     * Setea el area seleccionable del componente en base a los puntos que lo
     * conforman.
     */
    private void setArea() {
        double maxX = a.getX() + 1;
        double maxY = a.getY() + 1;
        double minX = a.getX() - 1;
        double minY = a.getY() - 1;
        double ancho = maxX - minX + 1;
        double alto = maxY - minY + 1;
        area = new Rectangle2D.Double(minX, minY, ancho, alto);
        for (int i = 0; i < cables.size(); i++) {
            cables.get(i).setArea();
        }
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) {
        /*
         * Se traslada el punto central del Nodo con respecto al punto
         * "centro" en funcion del factor.
         */
        double nx;
        double ny;
        nx = (a.getX() - centro.getX()) * factor + centro.getX();
        ny = (a.getY() - centro.getY()) * factor + centro.getY();
        a.setLocation(nx, ny);
        setArea();
    }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Se uso la transformacion rotacional para un vector.
         * http://es.wikipedia.org/wiki/Rotaci%C3%B3n
         */
        double nx, ny;
        nx = (a.getX() - centro.getX()) * Math.cos(deltaAngulo)
            - (a.getY() - centro.getY()) * Math.sin(deltaAngulo);
        ny = (a.getX() - centro.getX()) * Math.sin(deltaAngulo)
            + (a.getY() - centro.getY()) * Math.cos(deltaAngulo);
        a.setLocation(nx + centro.getX(), ny + centro.getY());
        setArea();
    }

    /** Rota la forma respecto a su centro en 90 grados.
     */
    public void rotar() { }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Se translado cada coordenada en su delta respectivo.
         */
        a.setLocation(a.getX() + deltaX, a.getY() + deltaY);
        setArea();
    }

    /**
     * Verifica si un punto esta dentro de el area seleccionable de la forma.
     * @param p Punto a verificar.
     * @return Verdadero si el punto esta dentro del area.
     */
    public boolean isSelected(Point p) {
        if (area.contains(p)) {
            c = Color.RED;
            return true;
        }
        c = Color.BLUE;
        return false;
    }

    /** Dibuja la Forma en el Graphics2D.
     * @param g2d Grafico en el cual se incluira el dibujo.
     */
    public void paintComponent(Graphics2D g2d) {
        g2d.setColor(c);
        g2d.fill(area);
    }

    /** Retorna los nodos relacionados a este Componente.
     * @return Arreglo de nodos.
     */
    public Nodo[] getNodos() {
        Nodo[] nodos = new Nodo[1];
        nodos[0] = this;
        return nodos;
    }

    /**
     * Agrega un cable al nodo.
     * @param cable Cable a agregar.
     */
    public void addCable(Cable cable) {
        cables.add(cable);
    }

    /**
     * Quita un cable del nodo.
     * @param cable Cable a remover.
     */
    public void removeCable(Cable cable) {
        cables.remove(cable);
    }

    /** Retorna el punto que basta para localizar la coordenada de un
     * componente, no asi su orientacion.
     * @return Punto con las coordenadas del componente.
     */
    public Point getPunto() {
        return a;
    }

    /**
     * Retorna el area de este Nodo.
     * @return Area elegible de este nodo.
     */
    public Rectangle2D getArea() {
        return area;
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new Nodo(a);
    }

    /**
     * Reviza si existe interseccion de otro nodo con este.
     * @param n Nodo a comparar.
     * @return Verdadero si las areas de los nodos se intersectan.
     */
    public boolean intersects(Nodo n) {
        if (n.getArea().intersects(area)) {
            return true;
        }
        return false;
    }

    /**
     * Retorna el arreglo de cables unidos a este nodo.
     * @return Arreglo de cables.
     */
    public ArrayList < Cable > getCables() {
        return cables;
    }
}
